/*******************************************************************************
 *  Cirrus Logic fb operators
 *******************************************************************************
 *
   Copyright (C) 1997, The Linux-GGI Project
 *
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************
 *
 *  This code was derived from the following sources of information:
 *  [1]     Xfree 86 VGA driver
 *  [2]     Zen of Graphics Programming (Michael Abrash)
 */


static void cirrus_set_origin(struct kgi_display *dpy, kgi_uint x, kgi_uint y)
{
	IS_CRITICAL;
	static kgi_uint last_xdot = -1;
	static kgi_uint last_ydot = -1;
	static kgi_uint old_offset = -1;
	register kgi_uint xpix=0,xdot=0;
	register kgi_uint ypix=0,ydot=0;
	register kgi_uint offset;
	kgi_uint fb_width=0;

	switch (dpy->mode.request.graphtype) {
		case KGIGT_TEXT16:
			xpix = x / dpy->mode.request.dpp.x;
			xdot = x % dpy->mode.request.dpp.x;
			ypix = y / dpy->mode.request.dpp.y;
			ydot = y % dpy->mode.request.dpp.y;
			fb_width = dpy->mode.request.virt.x * 2;
			break;
		case KGIGT_1BIT:
		case KGIGT_4BIT:
			xpix = x / 8;
			xdot = x % 8;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x / 8;
			break;
		case KGIGT_8BIT:
			xpix = x;
			xdot = 0;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x;
			break;	
		case KGIGT_15BIT:
		case KGIGT_16BIT:
			xpix = x;
			xdot = 0;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x*2;
			break;
		case KGIGT_24BIT:
			xpix = x;
			xdot = 0;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x*3;
			break;
		case KGIGT_32BIT:
			xpix = x;
			xdot = 0;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x*4;
			break;
		default:
			INTERNAL_ERROR;
	}
		
 	offset = xpix + ypix * fb_width;

	ENTER_CRITICAL;

	if (old_offset != offset) {

		outbCRT(offset >> 8, CRT_STARTADDR_H);
		outbCRT(offset, CRT_STARTADDR_L);
		outbCRT((inbCRT(0x1B) & 0xF2) |
			((offset>>16) & 0x01) |
			((offset>>15) & 0x0C) , 0x1B);
		old_offset = offset;
	}

	if (last_ydot != ydot) {
		outbCRT(ydot & CR08_ROWSCAN_MASK, CRT_PRESETROW);
		last_ydot = ydot;
	}

	if (last_xdot != xdot) {

		register kgi_uint hpan;

		if (dpy->mode.request.graphtype == KGIGT_TEXT16 &&
		    dpy->mode.request.dpp.x == 9) {
			
			hpan = (xdot == 0) ? 0x08 : (xdot-1) & AR13_DATA_MASK;
		} else {
			
			hpan = xdot & AR13_DATA_MASK;
		}
		WAIT_HVRETRACE();
		outbATC(hpan, ATC_HPAN);
		outb( ATCI_ENABLEDISPLAY, ATCI);
		last_xdot = xdot;
	}
	
	LEAVE_CRITICAL;
}

static void cirrus_set_split(struct kgi_display *dpy, int line)
{
	outbCRT(( line        & 0xFF) , 0x18);

	outbCRT((inbCRT(0x07) & 0xEF) | 
		((line  >> 4) & 0x10) , 0x07);

	outbCRT((inbCRT(0x09) & 0xBF) | 
		((line >>  3) & 0x40) , 0x09);
}

#if 0
static void cirrus_set_pointer_shape(struct kgi_display *dpy,struct kgi_pointer64x64 *shape)
{
	switch(shape->mode)
	{
	case pmWindows:
	case pmX11:
	case pmThreeColor:
	default:
	}
}

static void cirrus_show_pointer(struct kgi_display *dpy,int x,int y)
{
	outw((x<<5)|0x10,0x3C4);
	outw((y<<5)|0x11,0x3C4);
	outw(0x8312,0x3C4);
}

static void cirrus_hide_pointer(struct kgi_display *dpy)
{
	outbSEQ(inbSEQ(0x12)&~1,0x12);
}

#define cirrus_undo_pointer NULL;
#endif
